
/**
 * Contine metoda statica <b>inCuvinte</b>
 * folosita la traducere in cuvinte
 * a numerelor de pina la 27 cifre zecimale
 * 
 * @author Marius Joldos
 * @version 2009.09.10
 * 

 */
public class NumereInCuvinte
{
    static private final String[] cifre= 
    {
        "zero",  "unu", "doi", "trei",  "patru", "cinci", "sase",  "sapte", "opt",  "noua"
    };
    static private final String[] grupe =
    {
        "", "mii", "milioane", "miliarde", "trilioane", "catralioane", "quintilioane", "sextilioane", "septilioane",
        "octilioane"
    };
    static private final String[] zeceDouazeci =
    {
        "zece", "unsprezece", "douasprezece", "treisprezece", "patrusprezece", "cincisprezece",
        "sasesprezece", "saptesprezece", "optsprezece", "nouasprezece"
    };
    static private final String[] pesteDouazeci =
    {
        "", "una", "doua", "trei", "patru", "cinci", "sase", "sapte", "opt", "noua"
    };
    /**
     * Trateaza o grupa de una, doua sau trei cifre
     * @param String reprezinta gruparea de tradus in cuvinte  
     * @return reprezentarea in cuvinte
     */
    private static String getGrupa(String numar)
    {
        String s = "";
        int len = numar.length();
        
        if (numar.compareTo("000") == 0) return s;
        switch(len)
        {
            case 3:   // numar de 3 cifre
                    if (numar.charAt(len - 3) != '0')
                    {       
                        s += pesteDouazeci[Integer.parseInt(numar.substring(0, 1))] + " sute ";
                    };
            case 2:  // numar de 2 cifre
                    if (numar.charAt(len - 2) >= '2')
                    {       // peste 20
                        s += pesteDouazeci[Integer.parseInt(numar.substring(len - 2, len - 1))] + " zeci ";
                        if (numar.charAt(len - 1) > '0')
                            s += "si " + cifre[Integer.parseInt(numar.substring(len - 1, len))];
                        break;
                    }
                    else
                    if (numar.charAt(len - 2) >= '1' && numar.charAt(len - 2) < '2')
                    {    // intre 10 si 20
                        s += " " + zeceDouazeci[Integer.parseInt(numar.substring(len -2, len))-10];
                        break;
                    }
            case 1:
                     if (numar.charAt(len - 1) != '0')
                     { // intre 1 si 9
                         s += " " + cifre[Integer.parseInt(numar.substring(len - 1, len))];
                     }
             default:
                     break;
        }
       // System.out.println(numar +  ": " + s);
      return s;
    }

    /**
     * Traduce in cuvinte un numar exprimat in cifre arabe
     * @param numarul in cifre
     * @return reprezentarea ca String continind cuvinte a numarului
     */
    public static String inCuvinte( String numar)
    {
        String s = "";
        int nrGrupe = numar.length() / 3;
        int restGrupa = numar.length() %3;
        
        // verifica sa fie doar cifre
        for (int i =0; i < numar.length(); i++)
        {
            if (!Character.isDigit(numar.charAt(i)))
            {
                s = "Numar eronat: "+ numar + "\nCifra a " + (i + 1)  + "-a nu este zecimala";
                return s;
            }
        }        
        // verifica sa nu fie prea lung numarul
        if (nrGrupe > 9)
        {
            s = "Numar prea mare";
            return s;
        }
        if (restGrupa > 0)
        {
            s += getGrupa(numar.substring(0, restGrupa));
            s += " " + grupe[nrGrupe];
        }
        for (int i = 0; i < nrGrupe; i++)
        {
            // System.out.println("i=" + i + ": " + s);
            String grupa = getGrupa(numar.substring(restGrupa + 3 * i, restGrupa + 3 * (i + 1)));
            if (i == 0) s += " ";
            s += (grupa.isEmpty()? "": grupa +  " " + grupe[nrGrupe - i - 1] + " ");
        }
        return s;
    }
    /**
     * Metoda main este folosita la testare. Nu este absolut necesara
     * @param args[0] va contine numarul de tradus exprimat in cifre arabe
     */ 
    
    public static void main(String[] args)
    {
        if (args.length == 0)
        {
            System.err.println("Trebuie sa dati un numar in cifre arabe ca argument");
            return;
        }
        if (args[0].isEmpty()) 
            System.err.println("Nu ati dat nici un numar");
        else
            System.out.println(args[0] + " =" + NumereInCuvinte.inCuvinte(args[0]));
    }
}
